`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2020/05/08 19:19:56
// Design Name: 
// Module Name: speaker
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


module speaker(
    input clk,
    input rst_n,
    input pb_Do,
    input pb_Re,
    input pb_Mi,
    input vol_up,
    input vol_down,
    output [3:0]ssd_ctl,
    output [7:0]segs,
    output audio_mclk,
    output audio_lrck,
    output audio_sck,
    output audio_sdin
    );
    wire clk_100Hz;
    wire [1:0]ssd_ctl_en;
    wire pbDo_d, pbRe_d, pbMi_d, vol_up_d, vol_down_d;
    wire up_pulse, down_pulse;
    wire [15:0]amp1, amp2;
    wire [3:0]digit1, digit0;
    reg [21:0]note_div;
    wire [15:0]audio_in_left, audio_in_right;
    wire [3:0]ssd_in;

    freq_div U0(
        .clk(clk),
        .rst_n(rst_n),
        .clk_ctl(ssd_ctl_en),
        .clk_100Hz(clk_100Hz)
    );

    debounce U_Do(
        .pb_in(pb_Do),
        .clk(clk_100Hz),
        .rst_n(rst_n),
        .pb_debounced(pbDo_d)
    );

    debounce U_Re(
        .pb_in(pb_Re),
        .clk(clk_100Hz),
        .rst_n(rst_n),
        .pb_debounced(pbRe_d)
    );

    debounce U_Mi(
        .pb_in(pb_Mi),
        .clk(clk_100Hz),
        .rst_n(rst_n),
        .pb_debounced(pbMi_d)
    );

    debounce U_up(
        .pb_in(vol_up),
        .clk(clk_100Hz),
        .rst_n(rst_n),
        .pb_debounced(vol_up_d)
    );

    debounce U_down(
        .pb_in(vol_down),
        .clk(clk_100Hz),
        .rst_n(rst_n),
        .pb_debounced(vol_down_d)
    );

    one_pulse U_up_d(
        .in_trig(vol_up_d),
        .clk(clk_100Hz),
        .rst_n(rst_n),
        .one_pulse(up_pulse)
    );

    one_pulse U_down_d(
        .in_trig(vol_down_d),
        .clk(clk_100Hz),
        .rst_n(rst_n),
        .one_pulse(down_pulse)
    );

always@*
    case ({pbDo_d,pbRe_d,pbMi_d})
        3'b100: note_div = 22'd191571;
        3'b010: note_div = 22'd170648;
        3'b001: note_div = 22'd151515;
        default: note_div = 22'd0;
    endcase

    buzzer_ctl U_buzzer(
        .clk(clk),
        .rst_n(rst_n),
        .note_div(note_div),
        .amp1(amp1),
        .amp2(amp2),
        .audio_left(audio_in_left),
        .audio_right(audio_in_right)
    );

    speaker_ctl U_speaker(
        .clk(clk),
        .rst_n(rst_n),
        .audio_left(audio_in_left),
        .audio_right(audio_in_right),
        .audio_mclk(audio_mclk),
        .audio_lrck(audio_lrck),
        .audio_sck(audio_sck),
        .audio_sdin(audio_sdin)
    );

    amplitude U_amp(
        .up_pulse(up_pulse),
        .down_pulse(down_pulse),
        .clk(clk_100Hz),
        .rst_n(rst_n),
        .amp1(amp1),
        .amp2(amp2),
        .value1(digit1),
        .value0(digit0)
    );

    scan_ctl U7(
        .ssd_ctl_en(ssd_ctl_en),
        .in3(4'b1111),
        .in2(4'b1111),
        .in1(digit1),
        .in0(digit0),
        .ssd_ctl(ssd_ctl),
        .ssd_in(ssd_in)
    );

    display U8(
        .in(ssd_in),
        .segs(segs)
    );

endmodule
